// Preliminaries and common definitions.

HalfEdge halfEdges[];	// Representing our geometry
int tris[];				// Index buffer
vec3 verts[];			// Vertex buffer

int metadata[];			// Small buffer to hold metadata needed in shaders.

int triHalfEdges[];		// Mapping from triangle index to one half edge of the triangle
int triangleToRoot[];	// Mapping from triangle to "root" triangle of chart in union-find
int rootToChart[];		// Mapping from "root" triangle to chart
int vertToChart[];		// Mapping from vertex to chart; used by the vertex shader during shading and rasterization

struct Chart {
	vec2 min;
	vec2 max;
	ivec2 intMin;
	ivec2 intMax;
	vec2 offset;
	int originalTriangle;
	int flipped;
};
Chart charts[]; 		// Our actual charts

// Number and buffer starting point of charts at a given size in even and odd positions,
// indexed by chart width in pixels.
int chartCountAndOffsetPerSize[];
int chartIndicesSparse[]; 	 // Charts we are Cuckoo hashing into a fixed order, by size.
int chartIndicesCompacted[]; // Charts compacted, in size order.
int sortedCharts[];			 // Mapping from sorted index to chart ID

int prefixSum[];			 // Prefix sum of chart widths, used for packing.

int pivotIndices[];			 // Index of the "pivot" chart (i.e. the last chart) for each row, per parallel scale factor.
uint overfillScaleFactor[];  // Scale required to fit all rows in the atlas width, per parallel scale factor. Declared as uint to perform atomic operations.
ivec2 offsets[];			 // x, y offsets of charts in the atlas, per parallel scale factor.
int numPivotsAndOverfill[];  // Number of folding points and the max vertical height reached by charts, per parallel scale factor.
int frontLine[];			 // Total chart height at each x-position in the atlas during packing, per parallel scale factor.

// Walk a triangle halfEdge.
#define walkTriHalfEdges(_curEdge,_tri) \
  int _index = triHalfEdges[_tri]; \
  HalfEdge _curEdge = halfEdges[_index]; \
  for(int i=0; i < 3; i++, _index = _curEdge.next, _curEdge = halfEdges[_index])

int getConnectedTriangle(HalfEdge edge) {
  if (edge.twin != NO_EDGE) {
    return halfEdges[edge.twin].face;
  } else return NO_TRIANGLE;
}

// Many of our algorithms index a chart by which scale it is being tested at.
#define CHARTINDEX(i, scale) (i + MAX_CHARTS * (scale))
